## hatebu.rb

require 'simple-json-parser-builder.rb'
require 'rexml/document'
require 'net/http'
require 'nkf'
require 'time'


module Hatebu
  module Utility
    self.extend( self )

    def enc( str )
      NKF.nkf( '-E -w', str )
    end

    def dec( str )
      NKF.nkf( '-W -e', str )
    end

    HTTP_GET_CALLBACK_ARRAY = [lambda {|url| puts "#{url}" }]
    def http_get( url )
      HTTP_GET_CALLBACK_ARRAY.each{|proc| proc.call( url ) }
      Net::HTTP.get( URI.parse( url ) )
    end

    def make_bookmark_url( url )
      "http://b.hatena.ne.jp/entry/" + url.gsub( /\#/, "%23" ).gsub( "https?\:\/\/", "" )
    end

    def make_bookmark_image_url( url )
      "http://b.hatena.ne.jp/entry/image/" + url.gsub( /\#/, "%23" )
    end
  end

  ## -- User -------------------------------------------------------------
  class User
    def initialize( name )
      @name = name
    end
    attr_reader( :name, :bookmark_count, :site, :favorite_count, :follower_count )

    def atom_feed_url( start_count = 0 )
      "http://b.hatena.ne.jp/#{@name}/atomfeed?of=#{start_count}"
    end

    def rss_feed_url
      "http://b.hatena.ne.jp/#{@name}/rss"
    end

    def sidebar_url
      "http://b.hatena.ne.jp/#{@name}/sidebar"
    end

    STOP_GET_BOOKMARK = Object.new
    NO_NEXT_ACCESS = Object.new

    def get_bookmark( start_count = 0 )
      feed_url = self.atom_feed_url( start_count )

      all = []
      done = {}
      loop {
        ary = []
        doc = REXML::Document.new( Utility.http_get( feed_url ) )
        next_url = nil
        doc.elements.each( '/feed/link' ) {|one|
          next_url = one.attribute( 'href' ).value if one.attribute( 'rel' ).value == 'next'
        }
        doc.elements.each( '/feed/entry' ) {|one|
          bookmark = Bookmark.new_from_atom( self, one )
          next if done.has_key?( bookmark.bookmark_id )
          stop_flag = yield bookmark
          all << bookmark
          done[bookmark.bookmark_id] = true
          return all if stop_flag == STOP_GET_BOOKMARK
          next_url = nil if stop_flag == NO_NEXT_ACCESS
        }
        break if next_url.nil?
        feed_url = next_url
      }
      all
    end

    def get_bookmark_count
      doc = REXML::Document.new( Utility.http_get( self.rss_feed_url ) )
      @bookmark_count = doc.elements['rdf:RDF/channel/opensearch:totalResults'].text.to_i
      @bookmark_count
    end

    def get_side_data
      data = Utility.dec( Utility.http_get( self.sidebar_url ) )

      reg = Regexp.compile( Regexp.escape( '֥åޡ</span>' ) + "\s*([^\<]*)" )
      m = reg.match( data )
      @bookmark_count = m.nil? ? nil : m[1].gsub( /\,/, '' ).to_i

      reg = Regexp.compile( Regexp.escape( '</span>' ) + '\s*<a href="([^\"]*)' )
      m = reg.match( data )
      @site = m.nil? ? nil : m[1]

      reg = Regexp.compile( "<a href=\"/#{name}/favorite\">([^\<]*)" )
      m = reg.match( data )
      @favorite_count = m.nil? ? nil : m[1].gsub( /\,/, '' ).to_i

      reg = Regexp.compile( "<a href=\"/#{name}/follower\">([^\<]*)" )
      m = reg.match( data )
      @follower_count = m.nil? ? nil : m[1].gsub( /\,/, '' ).to_i

      [@site, @bookmark_count, @favorite_count, @follower_count]
    end

  end

  ## -- Bookmark ---------------------------------------------------------
  class Bookmark
    def initialize( user )
      @user = user
    end

    attr_reader(
      :user,
      :bookmark_id,
      :title,
      :time,
      :comment,
      :target_url,
      :permalink_url,
      :tags
    )

    def self.new_from_atom( user, bookmark_element )
      obj = self.new( user )
      obj.initialize_from_atom( bookmark_element )
      obj
    end

    def initialize_from_atom( bookmark_element )
      get_text = lambda {|key| bookmark_element.elements[key].nil? ? nil : bookmark_element.elements[key].text }

      @bookmark_id = get_text.call( 'id' )
      @title       = get_text.call( 'title' )
      @comment     = get_text.call( 'summary' )
      self.set_time_from_text( get_text.call( 'issued' ) )

      bookmark_element.each_element( 'link' ) {|link_element|
        case link_element.attribute( 'rel' ).value
        when 'related'
          @target_url = link_element.attribute( 'href' ).value
        when 'alternate'
          @permalink_url = link_element.attribute( 'href' ).value
        end
      }

      @tags = []
      bookmark_element.elements.each( 'dc:subject' ) {|tag|
        @tags << tag.text
      }
    end

    def set_time_from_text( text )
      @time = Time.parse( text ) unless text.nil?
    end

    def self.new_from_json( data, target_url, title )
      obj = self.new( User.new( data['user'] ) )
      obj.initialize_from_json( data, target_url, title )
      obj
    end

    def initialize_from_json( data, target_url, title )
      get_data = lambda {|key| data.has_key?( key ) ? data[key] : nil }

      @bookmark_id   = nil
      @title         = title
      @comment       = get_data.call( 'comment' )
      @target_url    = target_url
      @permalink_url = nil
      @tags          = get_data.call( 'tags' )
      self.set_time_from_text( get_data.call( 'timestamp' ) )
    end

  end

  ## -- Entry ------------------------------------------------------------
  class Entry
    def self.get_bookmark_count( url )
      Utility.http_get( 'http://api.b.st-hatena.com/entry.count?url=' + url ).to_i
    end

    def initialize( target_url )
      @target_url = target_url
      @bookmarks = []
    end
    attr_reader(
      :target_url,
      :title,
      :bookmark_count,
      :entry_url,
      :screenshot_url,
      :entry_id,
      :bookmarks
    )

    def json_lite_url
      'http://b.hatena.ne.jp/entry/jsonlite/?url=' << @target_url.gsub( /\#/, '%23' ).gsub( /\&/, '%26' )
    end

    def get_data
      data = JsonParser.new.parse( Utility.http_get( self.json_lite_url ) )

      get_data = lambda{|key| data.has_key?( key ) ? data[key] : nil }

      @title          = get_data.call( 'title' )
      @bookmark_count = get_data.call( 'count' ).to_i
      @entry_url      = get_data.call( 'entry_url' )
      @screenshot_url = get_data.call( 'screenshot' )
      @entry_id       = get_data.call( 'eid' )
      @bookmarks = []
      if data.has_key?( 'bookmarks' )
        data['bookmarks'].reverse_each{|b|
          @bookmarks << Bookmark.new_from_json( b, @target_url, @title )
        }
      end
    end
  end

end
