## TTRL : releaser.rb

require 'fileutils'
require 'Win32API'

module TTRL
  class Releaser
    def initialize( name, base, test_mode = false )
      @name = name
      @base = base
      @files = {}
      @dirs = {}
      @pre_copy_procs = []
      @post_copy_procs = []
      @test_mode = test_mode
    end

    def add_file( file, to = "./" )
      @files[file] = to
    end

    def add_dir( dir, to = "./" )
      @dirs[dir] = to
    end

    def add_pre_copy_proc( proc )
      @pre_copy_procs << proc
    end

    def add_post_copy_proc( proc )
      @post_copy_procs << proc
    end

    def add_pre_copy( &block )
      self.add_pre_copy_proc( block )
    end

    def add_post_copy( &block )
      self.add_post_copy_proc( block )
    end

    def make_zip( release_dir, additional_string )
      today = Time.now.strftime( '%y%m%d' )
      file_utils = @test_mode ? FileUtils::DryRun : FileUtils::Verbose

      zip_func = Win32API.new( 'ZIP32J.DLL', 'Zip', %w(l p p l), 'i' )

      Dir.chdir( @base ) {
        to_base = "./#{@name}"
        file_utils.mkdir( to_base )
        @pre_copy_procs.each {|proc| proc.call( file_utils, to_base ) }
        begin
          @files.each {|file, to|
            file_utils.copy( "./#{file}", "#{to_base}/#{to}" )
          }
          @dirs.each {|dir, to|
            file_utils.cp_r( "./#{dir}", "#{to_base}/#{to}" )
          }
          @post_copy_procs.each {|proc| proc.call( file_utils, to_base ) }

          command = "-r -q ./#{release_dir}/#{@name}#{additional_string.to_s}#{today}.zip #{to_base}"
          puts( "zip command : #{command}" )
          unless ( @test_mode )
            buf = "\0" * 65536
            ret = zip_func.call( 0, command, buf, buf.size )
            if ret != 0
              puts buf.strip
              raise
            end
          end
        ensure
          file_utils.rm_r( to_base )
        end
      }
    end

  end
end
