#! /usr/local/bin/ruby

require 'base.rb'
require 'analyze_villagers.rb'
require 'nocomment.rb'
require 'progress.rb'
require 'make_cache.rb'
require 'murahatebu.rb'


module MuraHatebu
  ## -- ProgressStarter --------------------------------------------------
  class ProgressStarter
    DB_KEY = 'progress_checked_time'
    def initialize( db )
      @db = db
      @db.transaction {
        @checked_time = @db.fetch( DB_KEY, self.make_checked_time( TODAY ) )
      }
      if @checked_time.keys.first.to_date < Time.now.to_date
        @checked_time = self.make_checked_time( TODAY )
      end
    end

    def make_checked_time( date )
      h = {}
      h[Time.mktime( date.year, date.month, date.day, 7, 0 )] = false
      h[Time.mktime( date.year, date.month, date.day, 11, 30 )] = false
      h[Time.mktime( date.year, date.month, date.day, 16, 50 )] = false
      h[Time.mktime( date.year, date.month, date.day, 21, 0 )] = false
      h
    end

    def check
      flag = false
      @checked_time.keys.sort.each {|time|
        break if time > Time.now
        next if @checked_time[time]
        @checked_time[time] = true
        flag = true
      }
      @db.transaction { @db[DB_KEY] = @checked_time }
      flag
    end
  end

  ## -- Updater ----------------------------------------------------------
  class Updater
    LAST_DATE_FILENAME = 'lastdate.dat'

    def start
      @self_dir = File.dirname( File.expand_path( __FILE__ ) )
      ## -- ʣưɻߡƱϢ³ɻ
      db = PStore.new( Util.filename_to_path( LAST_DATE_FILENAME ) )
      db.transaction {
        if db.root?( 'processing' )
          time = db['processing']
          if time.class == Time && Time.now - time < 60 * 30
            return
          else
            db.delete( 'processing' )
          end
        end
        db['processing'] = Time.now
      }

      begin
        db.transaction{ @lastdate = db.fetch( 'lastdate', nil ) }
        unless @lastdate == TODAY
          CacheMaker.new.start( TODAY - 3 )
          Manager.new.start
          AnalyzeManager.new.start
          NoCommentManager.new.start

          db.transaction { db['lastdate'] = TODAY }
        end

        progress_starter = ProgressStarter.new( db )
        if progress_starter.check
          ProgressManager.new.start
        end

      ensure
        db.transaction{ db.delete( 'processing' ) }
      end
    end

  end
end
## -- main ---------------------------------------------------------------
MuraHatebu::TODAY = Date.parse( ARGV[0] ) rescue Date.today
MuraHatebu::YESTADAY = MuraHatebu::TODAY - 1
MuraHatebu::Updater.new.start
